@@ -339,13 +339,58 @@ def edges_from_sparse_matrix(file_name, ix_converter=None, sep="\t"):
                        "as they were not finite".format(nan_values, total_values))
 
 
-def edges_dict(edges):
+def edges_dict_from_sparse(edges):
+    """Constructs edges_dict from edges parsed
+    from sparse format data.
+
+    Args:
+        edges (iterator): iterator over (sink, source, weight) tuples
+
+    Returns:
+        dict: Nexted dictionary that allows
+              queries like: dict[source][sink] = weight
+    """
     edges_d = defaultdict(dict)
     for source, sink, weight in edges:
         edges_d[source][sink] = weight
     return edges_d
 
 
+def edges_dict_from_fanc(hic):
+    """Constructs edges_dict from edges
+    present in a fanc hic object.
+
+    Args:
+        hic (fanc.Hic): Object created by fanc.load()
+
+    Returns:
+        dict: Nexted dictionary that allows
+              queries like: dict[source][sink] = weight
+    """
+    edges_dict = defaultdict(dict)
+    for e in hic.edges(lazy=True):
+        edges_dict[e.source][e.sink] = e.weight
+    return edges_dict
+
+
+def oe_edges_dict_from_fanc(hic):
+    """Constructs edges_dict from edges
+    present in a fanc hic object. Weights are
+    transformed to observed / expected values.
+
+    Args:
+        hic (fanc.Hic): Object created by fanc.load()
+
+    Returns:
+        dict: Nexted dictionary that allows
+              queries like: dict[source][sink] = weight
+    """
+    edges_dict = defaultdict(dict)
+    for e in hic.edges(oe=True, lazy=True):
+        edges_dict[e.source][e.sink] = e.weight
+    return edges_dict
+
+
 def load_sparse_matrix(file_name, ix_converter=None, sep="\t"):
     return list(edges_from_sparse_matrix(file_name, ix_converter, sep))
 
@@ -435,6 +480,53 @@ def load_pairs_iter(file_name, sep=None):
             yield ID, cref, cqry
 
 
+def load_pairs_for_chroms(file_name, chromosomes, sep=None):
+    """Load region pairs from bedpe file.
+
+    Read input bedpe file and convert to dict mapping the comparison ID to the
+    reference to query regions. Return only pairs which are on the specified
+    chromosomes.
+
+    :param file_name: Path to input bedpe file. Expected columns:
+        chromosome_reference, start_reference, end_reference,
+        chromosome_query, start_query, end_query,
+        comparison_id, dummy column (not used), strand_reference, strand_query
+    :param sep: Delimiter in bedpe file, defaults to None (splits by tab)
+    :returns: Dict of form {ID: (reference_region, query_region)}, with
+              *region being :class:`GenomicRegion` objects.
+    """
+    pairs = set()
+    res = []
+    dropped = 0
+    with _open(file_name, 'r') as f:
+        for line in f:
+            if line.startswith('#'):
+                continue
+            sline = line.rstrip()
+            fields = sline.split(sep)
+            if len(fields) != 10:
+                raise ValueError((
+                    "10 columns expected but {} found"
+                    " in bedpe input"
+                    ).format(len(fields)))
+            c1, s1, e1, c2, s2, e2, ID, _, str1, str2 = line.split(sep)
+            if ID in pairs:
+                raise ValueError((
+                    "Pair ID {} not unique in bedpe input!"))
+            pairs.add(ID)
+            if any(c not in chromosomes for c in (c1, c2)):
+                dropped += 1
+            else:
+                cref = GenomicRegion(
+                    chromosome=str(c1), start=int(s1) + 1,
+                    end=int(e1), strand=str(str1))
+                cqry = GenomicRegion(
+                    chromosome=str(c2), start=int(s2) + 1,
+                    end=int(e2), strand=str(str2))
+                res.append((ID, cref, cqry))
+    return res, dropped
+
+
 def chunks(l, p):
     n = int(np.ceil(len(l)/p))
     """Yield successive len(l)/p-sized chunks from l."""
@@ -451,3 +543,56 @@ def read_chromosome_sizes(file_name):
                 chromosome, chromosome_length = line.split("\t")
                 chrom_sizes[chromosome] = int(chromosome_length)
     return chrom_sizes
+
+
+def load_contacts(matrix_file, regions_file=None):
+    import fanc
+    from chess.oe import observed_expected
+    try:
+        # try loading via fanc
+        reference_loaded = fanc.load(matrix_file)
+        edges = oe_edges_dict_from_fanc(reference_loaded)
+        regions = reference_loaded.regions
+        region_trees = region_interval_trees(
+            regions)
+    except ValueError as initial_error:
+        print(initial_error)
+        try:
+            assert regions_file is not None, (
+                "Regions file needs to be"
+                "specified for sparse input.")
+            regions, _ix_converter, _ = load_regions(
+                regions_file)
+            region_trees = region_interval_trees(
+                regions)
+            _, reference_oe = observed_expected(
+                regions_file, matrix_file)
+            edges = edges_dict_from_sparse(reference_oe)
+        except AssertionError as error:
+            raise ValueError(error)
+    return edges, region_trees, regions
+
+
+def load_oe_contacts(matrix_file, regions_file=None):
+    import fanc
+    try:
+        # try loading via fanc
+        reference_loaded = fanc.load(matrix_file)
+        edges = edges_dict_from_fanc(reference_loaded)
+        regions = reference_loaded.regions
+        region_trees = region_interval_trees(
+            regions)
+    except ValueError:
+        try:
+            assert regions_file is not None
+            regions, ix_converter, _ = load_regions(
+                regions_file)
+            region_trees = region_interval_trees(
+                regions)
+            edges = edges_dict_from_sparse(
+                edges_from_sparse_matrix(
+                    matrix_file,
+                    ix_converter))
+        except AssertionError:
+            raise ValueError
+    return edges, region_trees, regions
